/*******************************************************************************
* Copyright 2008 Seapine Software, Inc. All rights reserved.
*
* IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
* in consideration of your agreement to the following terms, and your use, installation,
* modification or redistribution of this Software constitutes acceptance of these terms.
* If you do not agree with these terms, please do not use, install, modify or redistribute
* this Software.
*
* In consideration of your agreement to abide by the following terms, and subject to these
* terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
* this original Seapine software (the Software), to use, reproduce, modify and redistribute
* the Software, with or without modifications, in source and/or binary forms; provided that
* if you redistribute the Software in its entirety and without modifications, you must retain
* this notice and the following text and disclaimers in all such redistributions of the Software.
* Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
* or promote products derived from the Software without specific prior written permission from
* Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
* implied, are granted by Seapine herein, including but not limited to any patent rights that
* may be infringed by your derivative works or by other works in which the Software may be
* incorporated.
*
* The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
* EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
* OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
*
* IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
* REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
* UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
* IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

namespace GenerateTRuns
{
    class Program
    {
        static void Main(string[] args)
        {
            //Before we do anything, let's make sure we have the nescessary arguments
            if (args.Length == 4)
            {
                //Define variables for app
                //TestTrack SOAP connection vars - passed as arguments
                string ttServer;
                string ttUser;
                string ttPassword;
                string createLink;

                //We'll get this from the environment variables DNUM and PROJNAME
                string ttDefNum;
                string ttProject;

                //This value is hard coded, but we could open defect and look for value in custom field
                string ttTestRunSet;

                //other variables for processing
                long ttCookie;
                long DNum = 0;

                

                //Set the variables to the corresponding arguments
                ttServer = args[0];
                ttUser = args[1];
                ttPassword = args[2];
                createLink = args[3];

                ttProject = System.Environment.GetEnvironmentVariable("PROJNAME");
                ttDefNum = System.Environment.GetEnvironmentVariable("DNUM");
                //ttProject = args[4];
                //ttDefNum = args[5];


                ttTestRunSet = "Alpha 1";

                try
                {
                    DNum = Convert.ToInt64(ttDefNum);
                }
                catch (Exception fcc)
                {
                    Logger.log("Unable to process the defect number.");
                    Logger.log("The following error was reported:");
                    Logger.log(fcc.Message);
                    Environment.Exit(0);
                }

                ttsoapcgi ttSoap = new ttsoapcgi();
                //Set the soap cgi URL
                ttSoap.Url = ttServer;
                //Get lists of projects for user and select one we want
                CProject ttCProject = null;
                try
                {
                    CProject[] ttCProjects = ttSoap.getProjectList(ttUser, ttPassword);
                    for (int i = 0; i < ttCProjects.Length; i++)
                    {
                        if (ttCProjects[i].database.name.Equals(ttProject))
                        {
                            ttCProject = ttCProjects[i];

                        }
                    }
                }
                catch (Exception fcc)
                {
                    Logger.log("Unable to retrieve lists of Projects:");
                    Logger.log(fcc.Message);
                    Environment.Exit(0);
                }
                if (ttCProject != null)
                {
                    try
                    {
                        //if we found project for user, lets log in
                        ttCookie = ttSoap.ProjectLogon(ttCProject, ttUser, ttPassword);
                        CDefect ttDefect = new CDefect();
                        //Get defect object that cuased this executable to run
                        ttDefect = ttSoap.getDefect(ttCookie, DNum, "", false);
                        //Get links from defect to see if there is a test case
                        CLink[] ttDLinks = ttSoap.getLinksForDefect(ttCookie, ttDefect.recordid);
                        //Loop through each link
                        for (int i = 0; i < ttDLinks.Length; i++)
                        {
                            //Loop through each member of each link
                            for (int j = 0; j < ttDLinks[i].childList.Length; j++)
                            {
                                //If link member is a test case, then we want it
                                
                                if (ttDLinks[i].childList[j].tablename == "Test Case")
                                {
                                    CLinkedItem ttLink = ttDLinks[i].childList[j];
                                    //Get test case object
                                    CTestCase ttTCase = ttSoap.getTestCase(ttCookie, Convert.ToInt64(ttLink.number), "", false);
                                    //before we can generate the test run for the test case, we need
                                    //to build our test runs.  We'll use all variants defined in the test case
                                    CTestRunVariantField[] ttRunVariants = new CTestRunVariantField[ttTCase.testVariants.Length];
                                    for (int k = 0; k < ttTCase.testVariants.Length; k++)
                                    {
                                        //Need to make sure we only look at the "Included" variants, and not the "Excluded" ones.
                                        if (ttTCase.testVariants[k].type == "Included")
                                        {
                                            CFieldValue[] tvVal = new CFieldValue[ttTCase.testVariants[k].values.Length];
                                            //Go through each value in this variant 
                                            for (int l = 0; l < ttTCase.testVariants[k].values.Length; l++)
                                            {
                                                tvVal[l] = new CFieldValue();
                                                tvVal[l].value = ttTCase.testVariants[k].values[l].value;
                                            }

                                            ttRunVariants[k] = new CTestRunVariantField();
                                            ttRunVariants[k].name = ttTCase.testVariants[k].name;
                                            ttRunVariants[k].values = tvVal;
                                        }
                                    }
                                    //Now we can generate the test run 
                                    
                                    CItemToTrack[] ttRuns = ttSoap.generateTestRuns(ttCookie, ttTCase.recordid, ttRunVariants, ttTestRunSet);
                                    //If yes to create link, then do it
                                    if (createLink.Equals("true"))
                                    {
                                        //Define the parent - Defect
                                        CLinkedItem parentLink = new CLinkedItem();
                                        parentLink.entityType = 0x64666374;
                                        parentLink.entityID = ttDefect.recordid;
                                        parentLink.number = ttDefNum;
                                        parentLink.tablename = "Defect";
                                        //For each test run created add to link
                                        CLinkedItem[] alinks = new CLinkedItem[ttRuns.Length];
                                        for (int m = 0; m < ttRuns.Length; m++)
                                        {
                                            //get the test run object
                                            CTestRun ttRun = ttSoap.getTestRunByRecordID(ttCookie, ttRuns[m].recordid, false);
                                            //Define the child - TestRun
                                            alinks[m] = new CLinkedItem();
                                            alinks[m].entityType = 0x64666374;
                                            alinks[m].entityID = ttRun.recordid;
                                            alinks[m].number = ttRun.testrunnumber.ToString();
                                            alinks[m].tablename = "Test Run";
                                        }
                                        //Create the link
                                        CLink link = new CLink();
                                        link.linkDefinitionName = "Parent/Child";
                                        link.linkparent = parentLink;
                                        link.childList = alinks;
                                        //add the link
                                        ttSoap.addLink(ttCookie, link);
                                    }
                                }
                            }
                        }
                        ttSoap.DatabaseLogoff(ttCookie);
                    }
                    catch (Exception fcc)
                    {
                        Logger.log("An error was encountered while processing the generation of test runs:");
                        Logger.log(fcc.Message);
                        Environment.Exit(0);
                    }
                }
            }
            else
            {
                Logger.log("Incorrect number of parameters passed");
                Logger.log(args.Length.ToString() + " parameter(s) were passed");
                Environment.Exit(0);
            }
        }
    }
}
